@@ -5,7 +5,7 @@
  */
 
 mergeInto(LibraryManager.library, {
-  $FS__deps: ['$setErrNo', '$getRandomDevice', '$PATH', '$PATH_FS', '$TTY', '$MEMFS',
+  $FS__deps: ['$getRandomDevice', '$PATH', '$PATH_FS', '$TTY', '$MEMFS',
 #if LibraryManager.has('library_idbfs.js')
     '$IDBFS',
 #endif
@@ -18,6 +18,9 @@ mergeInto(LibraryManager.library, {
 #if LibraryManager.has('library_noderawfs.js')
     '$NODERAWFS',
 #endif
+#if LibraryManager.has('library_proxyfs.js')
+    '$PROXYFS',
+#endif
 #if ASSERTIONS
     '$ERRNO_MESSAGES', '$ERRNO_CODES',
 #endif
@@ -109,11 +112,6 @@ FS.staticInit();` +
     filesystems: null,
     syncFSRequests: 0, // we warn if there are multiple in flight at once
 
-    handleFSError: function(e) {
-      if (!(e instanceof FS.ErrnoError)) throw e + ' : ' + stackTrace();
-      return setErrNo(e.errno);
-    },
-
     //
     // paths
     //
@@ -294,21 +292,15 @@ FS.staticInit();` +
     // permissions
     //
     flagModes: {
+      // Extra quotes used here on the keys to this object otherwise jsifier will
+      // erase them in the process of reading and then writing the JS library
+      // code.
       '"r"': {{{ cDefine('O_RDONLY') }}},
-      '"rs"': {{{ cDefine('O_RDONLY') }}} | {{{ cDefine('O_SYNC') }}},
       '"r+"': {{{ cDefine('O_RDWR') }}},
       '"w"': {{{ cDefine('O_TRUNC') }}} | {{{ cDefine('O_CREAT') }}} | {{{ cDefine('O_WRONLY') }}},
-      '"wx"': {{{ cDefine('O_TRUNC') }}} | {{{ cDefine('O_CREAT') }}} | {{{ cDefine('O_WRONLY') }}} | {{{ cDefine('O_EXCL') }}},
-      '"xw"': {{{ cDefine('O_TRUNC') }}} | {{{ cDefine('O_CREAT') }}} | {{{ cDefine('O_WRONLY') }}} | {{{ cDefine('O_EXCL') }}},
       '"w+"': {{{ cDefine('O_TRUNC') }}} | {{{ cDefine('O_CREAT') }}} | {{{ cDefine('O_RDWR') }}},
-      '"wx+"': {{{ cDefine('O_TRUNC') }}} | {{{ cDefine('O_CREAT') }}} | {{{ cDefine('O_RDWR') }}} | {{{ cDefine('O_EXCL') }}},
-      '"xw+"': {{{ cDefine('O_TRUNC') }}} | {{{ cDefine('O_CREAT') }}} | {{{ cDefine('O_RDWR') }}} | {{{ cDefine('O_EXCL') }}},
       '"a"': {{{ cDefine('O_APPEND') }}} | {{{ cDefine('O_CREAT') }}} | {{{ cDefine('O_WRONLY') }}},
-      '"ax"': {{{ cDefine('O_APPEND') }}} | {{{ cDefine('O_CREAT') }}} | {{{ cDefine('O_WRONLY') }}} | {{{ cDefine('O_EXCL') }}},
-      '"xa"': {{{ cDefine('O_APPEND') }}} | {{{ cDefine('O_CREAT') }}} | {{{ cDefine('O_WRONLY') }}} | {{{ cDefine('O_EXCL') }}},
       '"a+"': {{{ cDefine('O_APPEND') }}} | {{{ cDefine('O_CREAT') }}} | {{{ cDefine('O_RDWR') }}},
-      '"ax+"': {{{ cDefine('O_APPEND') }}} | {{{ cDefine('O_CREAT') }}} | {{{ cDefine('O_RDWR') }}} | {{{ cDefine('O_EXCL') }}},
-      '"xa+"': {{{ cDefine('O_APPEND') }}} | {{{ cDefine('O_CREAT') }}} | {{{ cDefine('O_RDWR') }}} | {{{ cDefine('O_EXCL') }}}
     },
     // convert the 'r', 'r+', etc. to it's corresponding set of O_* flags
     modeStringToFlags: function(str) {
@@ -1267,7 +1259,7 @@ FS.staticInit();` +
     },
     readFile: function(path, opts) {
       opts = opts || {};
-      opts.flags = opts.flags || 'r';
+      opts.flags = opts.flags || {{{ cDefine('O_RDONLY') }}};
       opts.encoding = opts.encoding || 'binary';
       if (opts.encoding !== 'utf8' && opts.encoding !== 'binary') {
         throw new Error('Invalid encoding type "' + opts.encoding + '"');
@@ -1288,7 +1280,7 @@ FS.staticInit();` +
     },
     writeFile: function(path, data, opts) {
       opts = opts || {};
-      opts.flags = opts.flags || 'w';
+      opts.flags = opts.flags || {{{ cDefine('O_TRUNC') | cDefine('O_CREAT') | cDefine('O_WRONLY') }}};
       var stream = FS.open(path, opts.flags, opts.mode);
       if (typeof data === 'string') {
         var buf = new Uint8Array(lengthBytesUTF8(data)+1);
@@ -1404,9 +1396,9 @@ FS.staticInit();` +
       }
 
       // open default streams for the stdin, stdout and stderr devices
-      var stdin = FS.open('/dev/stdin', 'r');
-      var stdout = FS.open('/dev/stdout', 'w');
-      var stderr = FS.open('/dev/stderr', 'w');
+      var stdin = FS.open('/dev/stdin', {{{ cDefine('O_RDONLY') }}});
+      var stdout = FS.open('/dev/stdout', {{{ cDefine('O_WRONLY') }}});
+      var stderr = FS.open('/dev/stderr', {{{ cDefine('O_WRONLY') }}});
 #if ASSERTIONS
       assert(stdin.fd === 0, 'invalid handle for stdin (' + stdin.fd + ')');
       assert(stdout.fd === 1, 'invalid handle for stdout (' + stdout.fd + ')');
@@ -1474,6 +1466,9 @@ FS.staticInit();` +
 #endif
 #if LibraryManager.has('library_workerfs.js')
         'WORKERFS': WORKERFS,
+#endif
+#if LibraryManager.has('library_proxyfs.js')
+        'PROXYFS': PROXYFS,
 #endif
       };
     },
@@ -1521,7 +1516,6 @@ FS.staticInit();` +
       if (ret.exists) {
         return ret.object;
       } else {
-        setErrNo(ret.error);
         return null;
       }
     },
@@ -1586,7 +1580,7 @@ FS.staticInit();` +
         }
         // make sure we can write to the file
         FS.chmod(node, mode | {{{ cDefine('S_IWUGO') }}});
-        var stream = FS.open(node, 'w');
+        var stream = FS.open(node, {{{ cDefine('O_TRUNC') | cDefine('O_CREAT') | cDefine('O_WRONLY') }}});
         FS.write(stream, data, 0, data.length, 0, canOwn);
         FS.close(stream);
         FS.chmod(node, mode);
@@ -1651,7 +1645,6 @@ FS.staticInit();` +
     // been loaded successfully. No-op for files that have been loaded already.
     forceLoadFile: function(obj) {
       if (obj.isDevice || obj.isFolder || obj.link || obj.contents) return true;
-      var success = true;
       if (typeof XMLHttpRequest !== 'undefined') {
         throw new Error("Lazy loading should have been performed (contents set) in createLazyFile, but it was not. Lazy loading only works in web workers. Use --embed-file or --preload-file in emcc on the main thread.");
       } else if (read_) {
@@ -1662,13 +1655,11 @@ FS.staticInit();` +
           obj.contents = intArrayFromString(read_(obj.url), true);
           obj.usedBytes = obj.contents.length;
         } catch (e) {
-          success = false;
+          throw new FS.ErrnoError({{{ cDefine('EIO') }}});
         }
       } else {
         throw new Error('Cannot load without read() or XMLHttpRequest.');
       }
-      if (!success) setErrNo({{{ cDefine('EIO') }}});
-      return success;
     },
     // Creates a file record for lazy-loading from a URL. XXX This requires a synchronous
     // XHR, which is not possible in browsers except in a web worker! Use preloading,
@@ -1807,17 +1798,13 @@ FS.staticInit();` +
       keys.forEach(function(key) {
         var fn = node.stream_ops[key];
         stream_ops[key] = function forceLoadLazyFile() {
-          if (!FS.forceLoadFile(node)) {
-            throw new FS.ErrnoError({{{ cDefine('EIO') }}});
-          }
+          FS.forceLoadFile(node);
           return fn.apply(null, arguments);
         };
       });
       // use a custom read function
       stream_ops.read = function stream_ops_read(stream, buffer, offset, length, position) {
-        if (!FS.forceLoadFile(node)) {
-          throw new FS.ErrnoError({{{ cDefine('EIO') }}});
-        }
+        FS.forceLoadFile(node);
         var contents = stream.node.contents;
         if (position >= contents.length)
           return 0;
