@@ -44,11 +44,6 @@ if (typeof WebAssembly !== 'object') {
 
 var wasmMemory;
 
-// In fastcomp asm.js, we don't need a wasm Table at all.
-// In the wasm backend, we polyfill the WebAssembly object,
-// so this creates a (non-native-wasm) table for us.
-#include "runtime_init_table.js"
-
 #if USE_PTHREADS
 // For sending to workers.
 var wasmModule;
@@ -187,117 +182,63 @@ function cwrap(ident, returnType, argTypes, opts) {
   }
 }
 
+#if ASSERTIONS
+// We used to include malloc/free by default in the past. Show a helpful error in
+// builds with assertions.
+#if !('_malloc' in IMPLEMENTED_FUNCTIONS)
+function _malloc() {
+  abort("malloc() called but not included in the build - add '_malloc' to EXPORTED_FUNCTIONS");
+}
+#endif // malloc
+#if !('_free' in IMPLEMENTED_FUNCTIONS)
+function _free() {
+  // Show a helpful error since we used to include free by default in the past.
+  abort("free() called but not included in the build - add '_free' to EXPORTED_FUNCTIONS");
+}
+#endif // free
+#endif // ASSERTIONS
+
 var ALLOC_NORMAL = 0; // Tries to use _malloc()
 var ALLOC_STACK = 1; // Lives for the duration of the current function call
-var ALLOC_DYNAMIC = 2; // Cannot be freed except through sbrk
-var ALLOC_NONE = 3; // Do not allocate
 
 // allocate(): This is for internal use. You can use it yourself as well, but the interface
 //             is a little tricky (see docs right below). The reason is that it is optimized
 //             for multiple syntaxes to save space in generated code. So you should
 //             normally not use allocate(), and instead allocate memory using _malloc(),
 //             initialize it with setValue(), and so forth.
-// @slab: An array of data, or a number. If a number, then the size of the block to allocate,
-//        in *bytes* (note that this is sometimes confusing: the next parameter does not
-//        affect this!)
-// @types: Either an array of types, one for each byte (or 0 if no type at that position),
-//         or a single type which is used for the entire block. This only matters if there
-//         is initial data - if @slab is a number, then this does not matter at all and is
-//         ignored.
+// @slab: An array of data.
 // @allocator: How to allocate memory, see ALLOC_*
-/** @type {function((TypedArray|Array<number>|number), string, number, number=)} */
-function allocate(slab, types, allocator, ptr) {
-  var zeroinit, size;
-  if (typeof slab === 'number') {
-    zeroinit = true;
-    size = slab;
-  } else {
-    zeroinit = false;
-    size = slab.length;
-  }
-
-  var singleType = typeof types === 'string' ? types : null;
-
+/** @type {function((Uint8Array|Array<number>), number)} */
+function allocate(slab, allocator) {
   var ret;
-  if (allocator == ALLOC_NONE) {
-    ret = ptr;
-  } else {
-    ret = [_malloc,
-#if DECLARE_ASM_MODULE_EXPORTS
-    stackAlloc,
-#else
-    typeof stackAlloc !== 'undefined' ? stackAlloc : null,
-#endif
-    dynamicAlloc][allocator](Math.max(size, singleType ? 1 : types.length));
-  }
-
-  if (zeroinit) {
-    var stop;
-    ptr = ret;
-    assert((ret & 3) == 0);
-    stop = ret + (size & ~3);
-    for (; ptr < stop; ptr += 4) {
-      {{{ makeSetValue('ptr', '0', '0', 'i32', null, true) }}};
-    }
-    stop = ret + size;
-    while (ptr < stop) {
-      {{{ makeSetValue('ptr++', '0', '0', 'i8', null, true) }}};
-    }
-    return ret;
-  }
-
-  if (singleType === 'i8') {
-    if (slab.subarray || slab.slice) {
-      HEAPU8.set(/** @type {!Uint8Array} */ (slab), ret);
-    } else {
-      HEAPU8.set(new Uint8Array(slab), ret);
-    }
-    return ret;
-  }
-
-  var i = 0, type, typeSize, previousType;
-  while (i < size) {
-    var curr = slab[i];
-
-    type = singleType || types[i];
-    if (type === 0) {
-      i++;
-      continue;
-    }
 #if ASSERTIONS
-    assert(type, 'Must know what type to store in allocate!');
+  assert(typeof allocator === 'number', 'allocate no longer takes a type argument')
+  assert(typeof slab !== 'number', 'allocate no longer takes a number as arg0')
 #endif
 
-    if (type == 'i64') type = 'i32'; // special case: we have one i32 here, and one i32 later
-
-    setValue(ret+i, curr, type);
-
-    // no need to look up size unless type changes, so cache it
-    if (previousType !== type) {
-      typeSize = getNativeTypeSize(type);
-      previousType = type;
-    }
-    i += typeSize;
+  if (allocator == ALLOC_STACK) {
+#if DECLARE_ASM_MODULE_EXPORTS
+    ret = stackAlloc(slab.length);
+#else
+    ret = (typeof stackAlloc !== 'undefined' ? stackAlloc : null)(slab.length);
+#endif
+  } else {
+    ret = {{{ makeMalloc('allocate', 'slab.length') }}};
   }
 
+  if (slab.subarray || slab.slice) {
+    HEAPU8.set(/** @type {!Uint8Array} */(slab), ret);
+  } else {
+    HEAPU8.set(new Uint8Array(slab), ret);
+  }
   return ret;
 }
 
-// Allocate memory during any stage of startup - static memory early on, dynamic memory later, malloc when ready
-function getMemory(size) {
-  if (!runtimeInitialized) return dynamicAlloc(size);
-  return _malloc(size);
-}
-
 #include "runtime_strings.js"
 #include "runtime_strings_extra.js"
 
 // Memory management
 
-var PAGE_SIZE = {{{ POSIX_PAGE_SIZE }}};
-var WASM_PAGE_SIZE = {{{ WASM_PAGE_SIZE }}};
-var ASMJS_PAGE_SIZE = {{{ ASMJS_PAGE_SIZE }}};
-
 function alignUp(x, multiple) {
   if (x % multiple > 0) {
     x += multiple - (x % multiple);
@@ -337,46 +278,26 @@ function updateGlobalBufferAndViews(buf) {
   Module['HEAPF64'] = HEAPF64 = new Float64Array(buf);
 }
 
-var STATIC_BASE = {{{ GLOBAL_BASE }}},
-    STACK_BASE = {{{ getQuoted('STACK_BASE') }}},
-    STACKTOP = STACK_BASE,
-    STACK_MAX = {{{ getQuoted('STACK_MAX') }}},
-    DYNAMIC_BASE = {{{ getQuoted('DYNAMIC_BASE') }}},
-    DYNAMICTOP_PTR = {{{ DYNAMICTOP_PTR }}};
+#if RELOCATABLE
+var __stack_pointer = new WebAssembly.Global({value: 'i32', mutable: true}, {{{ getQuoted('STACK_BASE') }}});
 
-#if ASSERTIONS
-assert(STACK_BASE % 16 === 0, 'stack must start aligned');
-assert(DYNAMIC_BASE % 16 === 0, 'heap must start aligned');
-#endif
-
-#if USE_PTHREADS
-if (ENVIRONMENT_IS_PTHREAD) {
-
-  // At the 'load' stage of Worker startup, we are just loading this script
-  // but not ready to run yet. At 'run' we receive proper values for the stack
-  // etc. and can launch a pthread. Set some fake values there meanwhile to
-  // catch bugs, then set the real values in establishStackSpace later.
-#if ASSERTIONS || STACK_OVERFLOW_CHECK >= 2
-  STACK_MAX = STACKTOP = STACK_MAX = 0x7FFFFFFF;
-#endif
-  // TODO DYNAMIC_BASE = Module['DYNAMIC_BASE'];
-  // TODO DYNAMICTOP_PTR = Module['DYNAMICTOP_PTR'];
-}
-#endif
+// To support such allocations during startup, track them on __heap_base and
+// then when the main module is loaded it reads that value and uses it to
+// initialize sbrk (the main module is relocatable itself, and so it does not
+// have __heap_base hardcoded into it - it receives it from JS as an extern
+// global, basically).
+Module['___heap_base'] = {{{ getQuoted('HEAP_BASE') }}};
+#endif // RELOCATABLE
 
 var TOTAL_STACK = {{{ TOTAL_STACK }}};
 #if ASSERTIONS
 if (Module['TOTAL_STACK']) assert(TOTAL_STACK === Module['TOTAL_STACK'], 'the stack size can no longer be determined at runtime')
 #endif
-#if MAIN_MODULE && !WASM
-// JS side modules use this value to decide their stack size.
-Module['TOTAL_STACK'] = TOTAL_STACK;
-#endif
 
-{{{ makeModuleReceiveWithVar('INITIAL_INITIAL_MEMORY', 'INITIAL_MEMORY', INITIAL_MEMORY) }}}
+{{{ makeModuleReceiveWithVar('INITIAL_MEMORY', 'INITIAL_MEMORY', INITIAL_MEMORY) }}}
 
 #if ASSERTIONS
-assert(INITIAL_INITIAL_MEMORY >= TOTAL_STACK, 'INITIAL_MEMORY should be larger than TOTAL_STACK, was ' + INITIAL_INITIAL_MEMORY + '! (TOTAL_STACK=' + TOTAL_STACK + ')');
+assert(INITIAL_MEMORY >= TOTAL_STACK, 'INITIAL_MEMORY should be larger than TOTAL_STACK, was ' + INITIAL_MEMORY + '! (TOTAL_STACK=' + TOTAL_STACK + ')');
 
 // check for full engine support (use string 'subarray' to avoid closure compiler confusion)
 assert(typeof Int32Array !== 'undefined' && typeof Float64Array !== 'undefined' && Int32Array.prototype.subarray !== undefined && Int32Array.prototype.set !== undefined,
@@ -404,8 +325,6 @@ if (typeof SharedArrayBuffer === 'undefined' || typeof Atomics === 'undefined')
 #endif
 #endif
 
-#include "runtime_sab_polyfill.js"
-
 #if STANDALONE_WASM
 #if ASSERTIONS
 // In standalone mode, the wasm creates the memory, and the user can't provide it.
@@ -416,29 +335,10 @@ assert(!Module['wasmMemory']);
 #include "runtime_init_memory.js"
 #endif // !STANDALONE_WASM
 
+#include "runtime_init_table.js"
 #include "runtime_stack_check.js"
 #include "runtime_assertions.js"
 
-function callRuntimeCallbacks(callbacks) {
-  while(callbacks.length > 0) {
-    var callback = callbacks.shift();
-    if (typeof callback == 'function') {
-      callback(Module); // Pass the module as the first argument.
-      continue;
-    }
-    var func = callback.func;
-    if (typeof func === 'number') {
-      if (callback.arg === undefined) {
-        Module['dynCall_v'](func);
-      } else {
-        Module['dynCall_vi'](func, callback.arg);
-      }
-    } else {
-      func(callback.arg === undefined ? null : callback.arg);
-    }
-  }
-}
-
 var __ATPRERUN__  = []; // functions called before the runtime is initialized
 var __ATINIT__    = []; // functions called during startup
 var __ATMAIN__    = []; // functions called when main() is to be run
@@ -477,6 +377,9 @@ function initRuntime() {
   assert(!runtimeInitialized);
 #endif
   runtimeInitialized = true;
+#if STACK_OVERFLOW_CHECK >= 2
+  Module['___set_stack_limits'](_emscripten_stack_get_base(), _emscripten_stack_get_end());
+#endif
   {{{ getQuoted('ATINITS') }}}
   callRuntimeCallbacks(__ATINIT__);
 }
@@ -551,11 +454,6 @@ function addOnPostRun(cb) {
   __ATPOSTRUN__.unshift(cb);
 }
 
-/** @param {number|boolean=} ignore */
-{{{ unSign }}}
-/** @param {number|boolean=} ignore */
-{{{ reSign }}}
-
 #include "runtime_math.js"
 
 // A counter of dependencies for calling run(). If we need to
@@ -661,8 +559,13 @@ function removeRunDependency(id) {
 
 Module["preloadedImages"] = {}; // maps url to image data
 Module["preloadedAudios"] = {}; // maps url to audio data
-#if WASM && MAIN_MODULE
+#if MAIN_MODULE
 Module["preloadedWasm"] = {}; // maps url to wasm instance exports
+addOnPreRun(preloadDylibs);
+#else
+#if RELOCATABLE
+addOnPreRun(reportUndefinedSymbols);
+#endif
 #endif
 
 /** @param {string|number=} what */
@@ -689,14 +592,10 @@ function abort(what) {
   what = output;
 #endif // ASSERTIONS
 
-#if WASM
   // Use a wasm runtime error, because a JS error might be seen as a foreign
   // exception, which means we'd run destructors on it. We need the error to
   // simply make the program stop.
   var e = new WebAssembly.RuntimeError(what);
-#else
-  var e = what;
-#endif
 
 #if MODULARIZE
   readyPromiseReject(e);
@@ -707,56 +606,7 @@ function abort(what) {
   throw e;
 }
 
-#if RELOCATABLE
-{{{
-(function() {
-  // add in RUNTIME_LINKED_LIBS, if provided
-  if (RUNTIME_LINKED_LIBS.length > 0) {
-    return "if (!Module['dynamicLibraries']) Module['dynamicLibraries'] = [];\n" +
-           "Module['dynamicLibraries'] = " + JSON.stringify(RUNTIME_LINKED_LIBS) + ".concat(Module['dynamicLibraries']);\n";
-  }
-  return '';
-})()
-}}}
-
-addOnPreRun(function() {
-  function loadDynamicLibraries(libs) {
-    if (libs) {
-      libs.forEach(function(lib) {
-        // libraries linked to main never go away
-        loadDynamicLibrary(lib, {global: true, nodelete: true});
-      });
-    }
-  }
-  // if we can load dynamic libraries synchronously, do so, otherwise, preload
-#if WASM
-  if (Module['dynamicLibraries'] && Module['dynamicLibraries'].length > 0 && !readBinary) {
-    // we can't read binary data synchronously, so preload
-    addRunDependency('preload_dynamicLibraries');
-    Promise.all(Module['dynamicLibraries'].map(function(lib) {
-      return loadDynamicLibrary(lib, {loadAsync: true, global: true, nodelete: true});
-    })).then(function() {
-      // we got them all, wonderful
-      removeRunDependency('preload_dynamicLibraries');
-    });
-    return;
-  }
-#endif
-  loadDynamicLibraries(Module['dynamicLibraries']);
-});
-
-#if ASSERTIONS
-function lookupSymbol(ptr) { // for a pointer, print out all symbols that resolve to it
-  var ret = [];
-  for (var i in Module) {
-    if (Module[i] === ptr) ret.push(i);
-  }
-  print(ptr + ' is ' + ret);
-}
-#endif
-#endif
-
-var memoryInitializer = null;
+// {{MEM_INITIALIZER}}
 
 #include "memoryprofiler.js"
 
@@ -802,7 +652,86 @@ function createExportWrapper(name, fixedasm) {
 }
 #endif
 
-#if WASM
+#if ABORT_ON_WASM_EXCEPTIONS
+// When DISABLE_EXCEPTION_CATCHING != 1 `abortWrapperDepth` counts the recursion
+// level of the wrapper function so that we only handle exceptions at the top level
+// letting the exception mechanics work uninterrupted at the inner level.
+// Additionally, `abortWrapperDepth` is also manually incremented in callMain so that
+// we know to ignore exceptions from there since they're handled by callMain directly.
+var abortWrapperDepth = 0;
+
+// Creates a wrapper in a closure so that each wrapper gets it's own copy of 'original'
+function makeAbortWrapper(original) {
+  return function() {
+    // Don't allow this function to be called if we're aborted!
+    if (ABORT) {
+      throw "program has already aborted!";
+    }
+
+#if DISABLE_EXCEPTION_CATCHING != 1
+    abortWrapperDepth += 1;
+#endif
+    try {
+      return original.apply(null, arguments);
+    }
+    catch (e) {
+      if (
+        ABORT // rethrow exception if abort() was called in the original function call above
+        || abortWrapperDepth > 1 // rethrow exceptions not caught at the top level if exception catching is enabled; rethrow from exceptions from within callMain
+#if SUPPORT_LONGJMP
+        || e === 'longjmp' // rethrow longjmp if enabled
+#endif
+      ) {
+        throw e;
+      }
+
+      abort("unhandled exception: " + [e, e.stack]);
+    }
+#if DISABLE_EXCEPTION_CATCHING != 1
+    finally {
+      abortWrapperDepth -= 1;
+    }
+#endif
+    }
+}
+
+// Instrument all the exported functions to:
+// - abort if an unhandled exception occurs
+// - throw an exception if someone tries to call them after the program has aborted
+// See settings.ABORT_ON_WASM_EXCEPTIONS for more info.
+function instrumentWasmExportsWithAbort(exports) {
+  // Override the exported functions with the wrappers and copy over any other symbols
+  var instExports = {};
+  for (var name in exports) {
+      var original = exports[name];
+      if (typeof original === 'function') {
+        instExports[name] = makeAbortWrapper(original);
+      } else {
+        instExports[name] = original;
+      }
+  }
+
+  return instExports;
+}
+
+function instrumentWasmTableWithAbort() {
+  // Override the wasmTable get function to return the wrappers
+  var realGet = wasmTable.get;
+  var wrapperCache = {};
+  wasmTable.get = function(i) {
+    var func = realGet.call(wasmTable, i);
+    var cached = wrapperCache[i];
+    if (!cached || cached.func !== func) {
+      cached = wrapperCache[i] = {
+        func: func,
+        wrapper: makeAbortWrapper(func)
+      }
+    }
+    return cached.wrapper;
+  };
+}
+#endif
+
 var wasmBinaryFile = '{{{ WASM_BINARY_FILE }}}';
 if (!isDataURI(wasmBinaryFile)) {
   wasmBinaryFile = locateFile(wasmBinaryFile);
@@ -876,8 +805,12 @@ function createWasm() {
     'a': asmLibraryArg,
 #else // MINIFY_WASM_IMPORTED_MODULES
     'env': asmLibraryArg,
-    '{{{ WASI_MODULE_NAME }}}': asmLibraryArg
+    '{{{ WASI_MODULE_NAME }}}': asmLibraryArg,
 #endif // MINIFY_WASM_IMPORTED_MODULES
+#if RELOCATABLE
+    'GOT.mem': new Proxy(asmLibraryArg, GOTHandler),
+    'GOT.func': new Proxy(asmLibraryArg, GOTHandler),
+#endif
   };
   // Load the wasm module and create an instance of using native support in the JS engine.
   // handle a generated wasm instance, receiving its exports and
@@ -885,13 +818,32 @@ function createWasm() {
   /** @param {WebAssembly.Module=} module*/
   function receiveInstance(instance, module) {
     var exports = instance.exports;
+
 #if RELOCATABLE
-    exports = relocateExports(exports, GLOBAL_BASE, 0);
+    exports = relocateExports(exports, {{{ GLOBAL_BASE }}});
 #endif
+
 #if ASYNCIFY
     exports = Asyncify.instrumentWasmExports(exports);
 #endif
+
+#if ABORT_ON_WASM_EXCEPTIONS
+    exports = instrumentWasmExportsWithAbort(exports);
+#endif
+
     Module['asm'] = exports;
+
+#if !RELOCATABLE
+    wasmTable = Module['asm']['__indirect_function_table'];
+#if ASSERTIONS && !PURE_WASI
+    assert(wasmTable, "table not found in wasm exports");
+#endif
+#endif
+
+#if ABORT_ON_WASM_EXCEPTIONS
+    instrumentWasmTableWithAbort();
+#endif
+
 #if !DECLARE_ASM_MODULE_EXPORTS
     // If we didn't declare the asm exports as top level enties this function
     // is in charge of programatically exporting them on the global object.
@@ -902,8 +854,11 @@ function createWasm() {
     // then exported.
     // TODO: do not create a Memory earlier in JS
     wasmMemory = exports['memory'];
-    updateGlobalBufferAndViews(wasmMemory.buffer);
 #if ASSERTIONS
+    assert(wasmMemory, "memory not found in wasm exports");
+#endif
+    updateGlobalBufferAndViews(wasmMemory.buffer);
+#if STACK_OVERFLOW_CHECK
     writeStackCookie();
 #endif
 #endif
@@ -1014,7 +969,7 @@ function createWasm() {
         !isFileURI(wasmBinaryFile) &&
 #endif
         typeof fetch === 'function') {
-      fetch(wasmBinaryFile, { credentials: 'same-origin' }).then(function (response) {
+      return fetch(wasmBinaryFile, { credentials: 'same-origin' }).then(function (response) {
         var result = WebAssembly.instantiateStreaming(response, info);
 #if USE_OFFSET_CONVERTER
         // This doesn't actually do another request, it only copies the Response object.
@@ -1022,6 +977,8 @@ function createWasm() {
         Promise.all([response.clone().arrayBuffer(), result]).then(function (results) {
           wasmOffsetConverter = new WasmOffsetConverter(new Uint8Array(results[0]), results[1].module);
           {{{ runOnMainThread("removeRunDependency('offset-converter');") }}}
+        }, function(reason) {
+          err('failed to initialize offset-converter: ' + reason);
         });
 #endif
         return result.then(receiveInstantiatedSource, function(reason) {
@@ -1118,7 +1075,12 @@ function createWasm() {
 #if RUNTIME_LOGGING
   err('asynchronously preparing wasm');
 #endif
+#if MODULARIZE
+  // If instantiation fails, reject the module ready promise.
+  instantiateAsync().catch(readyPromiseReject);
+#else
   instantiateAsync();
+#endif
 #if LOAD_SOURCE_MAP
   getSourceMapPromise().then(receiveSourceMapJSON);
 #endif
@@ -1128,7 +1090,6 @@ function createWasm() {
   return Module['asm']; // exports were assigned here
 #endif
 }
-#endif
 
 // Globals used by JS i64 conversions
 var tempDouble;
