@@ -298,6 +298,9 @@ var SYSCALL_DEBUG = 0;
 // Log out socket/network data transfer.
 var SOCKET_DEBUG = 0;
 
+// Log dynamic linker information
+var DYLINK_DEBUG = 0;
+
 // Select socket backend, either webrtc or websockets. XXX webrtc is not
 // currently tested, may be broken
 
@@ -585,6 +588,7 @@ var NODEJS_CATCH_EXIT = 1;
 // then exit with 0 (which hides the error if you don't read the log). With
 // this, we catch any unhandled rejection and throw an actual error, which will
 // make the process exit immediately with a non-0 return code.
+// This should be fixed in Node 15+.
 var NODEJS_CATCH_REJECTION = 1;
 
 // Whether to transform the code using asyncify. This makes it possible to
@@ -950,6 +954,10 @@ var DETERMINISTIC = 0;
 // resolved with the module instance when it is safe to run the compiled code,
 // similar to the `onRuntimeInitialized` callback. You do not need to use the
 // `onRuntimeInitialized` callback when using `MODULARIZE`.
+//
+// (If WASM_ASYNC_COMPILATION is off, that is, if compilation is
+// *synchronous*, then it would not make sense to return a Promise, and instead
+// the Module object itself is returned, which is ready to be used.)
 // 
 // The default name of the function is `Module`, but can be changed using the
 // `EXPORT_NAME` option. We recommend renaming it to a more typical name for a
@@ -1097,9 +1105,8 @@ var WASM = 1;
 // means we don't need to legalize for JS (but this requires a new enough JS
 // VM).
 //
-// Standlone builds by default require a `main` entry point.  If you want to
-// build a library (also known as a reactor) instead you can pass `--no-entry`
-// or specify a list of EXPORTED_FUNCTIONS that does not include `main`.
+// Standlone builds require a `main` entry point by default.  If you want to
+// build a library (also known as a reactor) instead you can pass `--no-entry`.
 var STANDALONE_WASM = 0;
 
 // Whether to ignore implicit traps when optimizing in binaryen.  Implicit
@@ -1615,6 +1622,14 @@ var ERROR_ON_WASM_CHANGES_AFTER_LINK = 0;
 // cost of a few bytes extra.
 var ABORT_ON_WASM_EXCEPTIONS = 0;
 
+// Build binaries that use as many WASI APIs as possible, and include additional
+// JS support libraries for those APIs.  This allows emscripten to produce binaries
+// are more WASI compilant and also allows it to process and execute WASI
+// binaries built with other SDKs (e.g.  wasi-sdk).
+// This setting is experimental and subject to change or removal.
+// Implies STANDALONE_WASM.
+var PURE_WASI = 0;
+
 //===========================================
 // Internal, used for testing only, from here
 //===========================================
@@ -1625,15 +1640,6 @@ var OFFSCREEN_FRAMEBUFFER_FORBID_VAO_PATH = 0;
 // Internal (testing only): Forces memory growing to fail.
 var TEST_MEMORY_GROWTH_FAILS = 0;
 
-// Advanced: Customize this array to reduce the set of asm.js runtime variables
-// that are generated. This allows minifying extra bit of asm.js code from unused
-// runtime code, if you know some of these are not needed.
-// (think of this as advanced manual DCE)
-// TODO(sbc): Move to settings_internal (current blocked due to use in test
-// code).
-var ASM_PRIMITIVE_VARS = ['__THREW__', 'threwValue', 'setjmpId', 'tempInt', 'tempBigInt', 'tempBigIntS', 'tempValue', 'tempDouble', 'tempFloat', 'tempDoublePtr', 'STACKTOP', 'STACK_MAX']
-
-// Legacy settings that have been removed or renamed.
 // For renamed settings the format is:
 // [OLD_NAME, NEW_NAME]
 // For removed settings (which now effectively have a fixed value and can no
@@ -1698,4 +1704,5 @@ var LEGACY_SETTINGS = [
   ['EXPORT_FUNCTION_TABLES', [0], 'No longer needed'],
   ['BINARYEN_SCRIPTS', [""], 'No longer needed'],
   ['WARN_UNALIGNED', [0, 1], 'No longer needed'],
+  ['ASM_PRIMITIVE_VARS', [[]], 'No longer needed'],
 ];
