@@ -10,8 +10,6 @@
 #include "../internal/pthread_impl.h"
 #include <assert.h>
 #include <dirent.h>
-#include <emscripten.h>
-#include <emscripten/threading.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <math.h>
@@ -30,6 +28,10 @@
 #include <unistd.h>
 #include <utime.h>
 
+#include <emscripten.h>
+#include <emscripten/threading.h>
+#include <emscripten/stack.h>
+
 // With LLVM 3.6, C11 is the default compilation mode.
 // gets() is deprecated under that standard, but emcc
 // still provides it, so always include it in the build.
@@ -938,10 +940,9 @@ int proxy_main(int argc, char** argv) {
   if (emscripten_has_threading_support()) {
     pthread_attr_t attr;
     pthread_attr_init(&attr);
-    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);
-    // Use TOTAL_STACK for the stack size, which is the normal size of the stack
+    // Use the size of the current stack, which is the normal size of the stack
     // that main() would have without PROXY_TO_PTHREAD.
-    pthread_attr_setstacksize(&attr, EM_ASM_INT({ return TOTAL_STACK }));
+    pthread_attr_setstacksize(&attr, emscripten_stack_get_base() - emscripten_stack_get_end());
     // Pass special ID -1 to the list of transferred canvases to denote that the thread creation
     // should instead take a list of canvases that are specified from the command line with
     // -s OFFSCREENCANVASES_TO_PTHREAD linker flag.
