@@ -3,8 +3,6 @@
 # University of Illinois/NCSA Open Source License.  Both these licenses can be
 # found in the LICENSE file.
 
-from __future__ import print_function
-
 import atexit
 import json
 import logging
@@ -22,16 +20,18 @@ from . import diagnostics
 from . import response_file
 from . import shared
 from . import webassembly
+from . import config
 from .toolchain_profiler import ToolchainProfiler
 from .shared import Settings, CLANG_CC, CLANG_CXX, PYTHON
-from .shared import LLVM_NM, EMCC, EMAR, EMXX, EMRANLIB, NODE_JS, WASM_LD, LLVM_AR
+from .shared import LLVM_NM, EMCC, EMAR, EMXX, EMRANLIB, WASM_LD, LLVM_AR
 from .shared import LLVM_LINK, LLVM_OBJCOPY
 from .shared import try_delete, run_process, check_call, exit_with_error
 from .shared import configuration, path_from_root, EXPECTED_BINARYEN_VERSION
-from .shared import asmjs_mangle, DEBUG, WINDOWS, JAVA
-from .shared import EM_BUILD_VERBOSE, TEMP_DIR, print_compiler_stage, BINARYEN_ROOT
+from .shared import asmjs_mangle, DEBUG
+from .shared import EM_BUILD_VERBOSE, TEMP_DIR, print_compiler_stage
 from .shared import CANONICAL_TEMP_DIR, LLVM_DWARFDUMP, demangle_c_symbol_name, asbytes
-from .shared import get_emscripten_temp_dir, exe_suffix, which, is_c_symbol
+from .shared import get_emscripten_temp_dir, exe_suffix, is_c_symbol
+from .utils import which, WINDOWS
 
 logger = logging.getLogger('building')
 
@@ -291,10 +291,10 @@ def handle_cmake_toolchain(args, env):
   # Append the Emscripten toolchain file if the user didn't specify one.
   if not has_substr(args, '-DCMAKE_TOOLCHAIN_FILE'):
     args.append('-DCMAKE_TOOLCHAIN_FILE=' + path_from_root('cmake', 'Modules', 'Platform', 'Emscripten.cmake'))
-  node_js = NODE_JS
+  node_js = config.NODE_JS
 
   if not has_substr(args, '-DCMAKE_CROSSCOMPILING_EMULATOR'):
-    node_js = NODE_JS[0].replace('"', '\"')
+    node_js = config.NODE_JS[0].replace('"', '\"')
     args.append('-DCMAKE_CROSSCOMPILING_EMULATOR="%s"' % node_js)
 
   # On Windows specify MinGW Makefiles or ninja if we have them and no other
@@ -335,7 +335,7 @@ def configure(args, stdout=None, stderr=None, env=None, cflags=[], **kwargs):
     stdout = None
   if EM_BUILD_VERBOSE >= 1:
     stderr = None
-  print('configure: ' + ' '.join(args), file=sys.stderr)
+  print('configure: ' + shared.shlex_join(args), file=sys.stderr)
   run_process(args, stdout=stdout, stderr=stderr, env=env, **kwargs)
 
 
@@ -438,8 +438,9 @@ def llvm_backend_args():
     allowed = ','.join(Settings.EXCEPTION_CATCHING_ALLOWED or ['__fake'])
     args += ['-emscripten-cxx-exceptions-allowed=' + allowed]
 
-  # asm.js-style setjmp/longjmp handling
-  args += ['-enable-emscripten-sjlj']
+  if Settings.SUPPORT_LONGJMP:
+    # asm.js-style setjmp/longjmp handling
+    args += ['-enable-emscripten-sjlj']
 
   # better (smaller, sometimes faster) codegen, see binaryen#1054
   # and https://bugs.llvm.org/show_bug.cgi?id=39488
@@ -482,9 +483,6 @@ def lld_flags_for_executable(external_symbol_list):
   # wasm module arrives)
   if not Settings.STANDALONE_WASM:
     cmd.append('--import-memory')
-    cmd.append('--import-table')
-  else:
-    cmd.append('--export-table')
 
   if Settings.USE_PTHREADS:
     cmd.append('--shared-memory')
@@ -503,6 +501,10 @@ def lld_flags_for_executable(external_symbol_list):
     else:
       cmd.append('--no-gc-sections')
       cmd.append('--export-dynamic')
+  else:
+    cmd.append('--export-table')
+    if Settings.ALLOW_TABLE_GROWTH:
+      cmd.append('--growable-table')
 
   if Settings.LINKABLE:
     cmd.append('--export-all')
@@ -813,10 +815,6 @@ def emar(action, output_filename, filenames, stdout=None, stderr=None, env=None)
     assert os.path.exists(output_filename), 'emar could not create output file: ' + output_filename
 
 
-def can_inline():
-  return Settings.INLINING_LIMIT == 0
-
-
 def get_safe_internalize():
   if Settings.LINKABLE:
     return [] # do not internalize anything
@@ -873,7 +871,7 @@ def acorn_optimizer(filename, passes, extra_info=None, return_output=False):
     with open(temp, 'a') as f:
       f.write('// EXTRA_INFO: ' + extra_info)
     filename = temp
-  cmd = NODE_JS + [optimizer, filename] + passes
+  cmd = config.NODE_JS + [optimizer, filename] + passes
   # Keep JS code comments intact through the acorn optimization pass so that JSDoc comments
   # will be carried over to a later Closure run.
   if Settings.USE_CLOSURE_COMPILER:
@@ -890,7 +888,7 @@ def acorn_optimizer(filename, passes, extra_info=None, return_output=False):
 
 # evals ctors. if binaryen_bin is provided, it is the dir of the binaryen tool
 # for this, and we are in wasm mode
-def eval_ctors(js_file, binary_file, binaryen_bin='', debug_info=False):
+def eval_ctors(js_file, binary_file, debug_info=False): # noqa
   logger.debug('Ctor evalling in the wasm backend is disabled due to https://github.com/emscripten-core/emscripten/issues/9527')
   return
   # TODO re-enable
@@ -903,7 +901,7 @@ def eval_ctors(js_file, binary_file, binaryen_bin='', debug_info=False):
 
 def get_closure_compiler():
   # First check if the user configured a specific CLOSURE_COMPILER in thier settings
-  if shared.CLOSURE_COMPILER:
+  if config.CLOSURE_COMPILER:
     return shared.CLOSURE_COMPILER
 
   # Otherwise use the one installed vai npm
@@ -916,16 +914,22 @@ def get_closure_compiler():
   return cmd
 
 
-def check_closure_compiler(cmd, args, env):
+def check_closure_compiler(cmd, args, env, allowed_to_fail):
   try:
     output = run_process(cmd + args + ['--version'], stdout=PIPE, env=env).stdout
   except Exception as e:
+    if allowed_to_fail:
+      return False
     logger.warn(str(e))
     exit_with_error('closure compiler ("%s --version") did not execute properly!' % str(cmd))
 
   if 'Version:' not in output:
+    if allowed_to_fail:
+      return False
     exit_with_error('unrecognized closure compiler --version output (%s):\n%s' % (str(cmd), output))
 
+  return True
+
 
 def closure_compiler(filename, pretty=True, advanced=True, extra_closure_args=None):
   with ToolchainProfiler.profile_block('closure_compiler'):
@@ -940,7 +944,7 @@ def closure_compiler(filename, pretty=True, advanced=True, extra_closure_args=No
     # Closure compiler expects JAVA_HOME to be set *and* java.exe to be in the PATH in order
     # to enable use the java backend.  Without this it will only try the native and JavaScript
     # versions of the compiler.
-    java_bin = os.path.dirname(JAVA)
+    java_bin = os.path.dirname(config.JAVA)
     if java_bin:
       def add_to_path(dirname):
         env['PATH'] = env['PATH'] + os.pathsep + dirname
@@ -948,13 +952,13 @@ def closure_compiler(filename, pretty=True, advanced=True, extra_closure_args=No
       java_home = os.path.dirname(java_bin)
       env.setdefault('JAVA_HOME', java_home)
 
-    if WINDOWS and not any(a.startswith('--platform') for a in user_args):
-      # Disable native compiler on windows until upstream issue is fixed:
-      # https://github.com/google/closure-compiler-npm/issues/147
-      user_args.append('--platform=java')
-
     closure_cmd = get_closure_compiler()
-    check_closure_compiler(closure_cmd, user_args, env)
+
+    native_closure_compiler_works = check_closure_compiler(closure_cmd, user_args, env, allowed_to_fail=True)
+    if not native_closure_compiler_works and not any(a.startswith('--platform') for a in user_args):
+      # Run with Java Closure compiler as a fallback if the native version does not work
+      user_args.append('--platform=java')
+      check_closure_compiler(closure_cmd, user_args, env, allowed_to_fail=False)
 
     # Closure externs file contains known symbols to be extern to the minification, Closure
     # should not minify these symbol names.
@@ -1002,7 +1006,7 @@ def closure_compiler(filename, pretty=True, advanced=True, extra_closure_args=No
 
     args = ['--compilation_level', 'ADVANCED_OPTIMIZATIONS' if advanced else 'SIMPLE_OPTIMIZATIONS']
     # Keep in sync with ecmaVersion in tools/acorn-optimizer.js
-    args += ['--language_in', 'ECMASCRIPT_2018']
+    args += ['--language_in', 'ECMASCRIPT_2020']
     # Tell closure not to do any transpiling or inject any polyfills.
     # At some point we may want to look into using this as way to convert to ES5 but
     # babel is perhaps a better tool for that.
@@ -1144,6 +1148,7 @@ def metadce(js_file, wasm_file, minify_whitespace, debug_info):
       'reaches': [],
       'root': True
     })
+  if not Settings.RELOCATABLE:
     graph.append({
       'export': '__indirect_function_table',
       'name': 'emcc$export$__indirect_function_table',
@@ -1162,6 +1167,8 @@ def metadce(js_file, wasm_file, minify_whitespace, debug_info):
     'fd_seek',
     'fd_fdstat_get',
     'fd_sync',
+    'fd_pread',
+    'fd_pwrite',
     'proc_exit',
     'clock_res_get',
     'clock_time_get',
@@ -1286,12 +1293,6 @@ def wasm2js(js_file, wasm_file, opt_level, minify_whitespace, use_closure_compil
   # JS optimizations
   if opt_level >= 2:
     passes = []
-    # it may be useful to also run: simplifyIfs, registerize, asmLastOpts
-    # passes += ['simplifyExpressions'] # XXX fails on wasm3js.test_sqlite
-    # TODO: enable name minification with pthreads. atm wasm2js emits pthread
-    # helper functions outside of the asmFunc(), and they mix up minifyGlobals
-    # (which assumes any vars in that area are global, like var HEAP8, but
-    # those helpers have internal vars in a scope it doesn't understand yet)
     if not debug_info and not Settings.USE_PTHREADS:
       passes += ['minifyNames']
     if minify_whitespace:
@@ -1363,7 +1364,13 @@ def emit_debug_on_side(wasm_file, wasm_file_with_dwarf):
   wasm_file_with_dwarf = shared.Settings.SEPARATE_DWARF
   if wasm_file_with_dwarf is True:
     wasm_file_with_dwarf = wasm_file + '.debug.wasm'
-  embedded_path = shared.Settings.SEPARATE_DWARF_URL or wasm_file_with_dwarf
+  embedded_path = shared.Settings.SEPARATE_DWARF_URL
+  if not embedded_path:
+    # a path was provided - make it relative to the wasm.
+    embedded_path = os.path.relpath(wasm_file_with_dwarf,
+                                    os.path.dirname(wasm_file))
+    # normalize the path to use URL-style separators, per the spec
+    embedded_path = embedded_path.replace('\\', '/').replace('//', '/')
 
   shutil.move(wasm_file, wasm_file_with_dwarf)
   strip(wasm_file_with_dwarf, wasm_file, debug=True)
@@ -1402,6 +1409,11 @@ def instrument_js_for_asan(js_file):
   return acorn_optimizer(js_file, ['asanify'])
 
 
+def instrument_js_for_safe_heap(js_file):
+  logger.debug('instrumenting JS memory accesses for SAFE_HEAP')
+  return acorn_optimizer(js_file, ['safeHeap'])
+
+
 def handle_final_wasm_symbols(wasm_file, symbols_file, debug_info):
   logger.debug('handle_final_wasm_symbols')
   args = []
@@ -1410,6 +1422,9 @@ def handle_final_wasm_symbols(wasm_file, symbols_file, debug_info):
   if not debug_info:
     # to remove debug info, we just write to that same file, and without -g
     args += ['-o', wasm_file]
+  else:
+    # suppress the wasm-opt warning regarding "no output file specified"
+    args += ['--quiet']
   # ignore stderr because if wasm-opt is run without a -o it will warn
   output = run_wasm_opt(wasm_file, args=args, stdout=PIPE)
   if symbols_file:
@@ -1456,48 +1471,67 @@ def is_wasm(filename):
 # Given the name of a special Emscripten-implemented system library, returns an
 # array of absolute paths to JS library files inside emscripten/src/ that
 # corresponds to the library name.
-def path_to_system_js_libraries(library_name):
+def map_to_js_libs(library_name):
   # Some native libraries are implemented in Emscripten as system side JS libraries
-  js_system_libraries = {
-    'c': '',
-    'dl': '',
-    'EGL': 'library_egl.js',
+  library_map = {
+    'c': [],
+    'dl': [],
+    'EGL': ['library_egl.js'],
     'GL': ['library_webgl.js', 'library_html5_webgl.js'],
     'webgl.js': ['library_webgl.js', 'library_html5_webgl.js'],
-    'GLESv2': 'library_webgl.js',
+    'GLESv2': ['library_webgl.js'],
     # N.b. there is no GLESv3 to link to (note [f] in https://www.khronos.org/registry/implementers_guide.html)
-    'GLEW': 'library_glew.js',
-    'glfw': 'library_glfw.js',
-    'glfw3': 'library_glfw.js',
-    'GLU': '',
-    'glut': 'library_glut.js',
-    'm': '',
-    'openal': 'library_openal.js',
-    'rt': '',
-    'pthread': '',
-    'X11': 'library_xlib.js',
-    'SDL': 'library_sdl.js',
-    'stdc++': '',
-    'uuid': 'library_uuid.js',
-    'websocket': 'library_websocket.js'
+    'GLEW': ['library_glew.js'],
+    'glfw': ['library_glfw.js'],
+    'glfw3': ['library_glfw.js'],
+    'GLU': [],
+    'glut': ['library_glut.js'],
+    'm': [],
+    'openal': ['library_openal.js'],
+    'rt': [],
+    'pthread': [],
+    'X11': ['library_xlib.js'],
+    'SDL': ['library_sdl.js'],
+    'stdc++': [],
+    'uuid': ['library_uuid.js'],
+    'websocket': ['library_websocket.js']
   }
-  library_files = []
-  if library_name in js_system_libraries:
-    if len(js_system_libraries[library_name]):
-      lib = js_system_libraries[library_name] if isinstance(js_system_libraries[library_name], list) else [js_system_libraries[library_name]]
-      library_files += lib
-      logger.debug('Linking in JS library ' + str(lib))
 
-  elif library_name.endswith('.js') and os.path.isfile(path_from_root('src', 'library_' + library_name)):
-    library_files += ['library_' + library_name]
+  if library_name in library_map:
+    libs = library_map[library_name]
+    logger.debug('Mapping library `%s` to JS libraries: %s' % (library_name, libs))
+    return libs
 
-  return library_files
+  if library_name.endswith('.js') and os.path.isfile(path_from_root('src', 'library_' + library_name)):
+    return ['library_' + library_name]
+
+  return None
 
 
-def emit_wasm_source_map(wasm_file, map_file):
+# map a linker flag to a Settings option, and apply it. this lets a user write
+# -lSDL2 and it will have the same effect as -s USE_SDL=2.
+def map_and_apply_to_settings(library_name):
+  # most libraries just work, because the -l name matches the name of the
+  # library we build. however, if a library has variations, which cause us to
+  # build multiple versions with multiple names, then we need this mechanism.
+  library_map = {
+    # SDL2_mixer's built library name contains the specific codecs built in.
+    'SDL2_mixer': [('USE_SDL_MIXER', 2)],
+  }
+
+  if library_name in library_map:
+    for key, value in library_map[library_name]:
+      logger.debug('Mapping library `%s` to settings changes: %s = %s' % (library_name, key, value))
+      setattr(shared.Settings, key, value)
+    return True
+
+  return False
+
+
+def emit_wasm_source_map(wasm_file, map_file, final_wasm):
   # source file paths must be relative to the location of the map (which is
   # emitted alongside the wasm)
-  base_path = os.path.dirname(os.path.abspath(Settings.WASM_BINARY_FILE))
+  base_path = os.path.dirname(os.path.abspath(final_wasm))
   sourcemap_cmd = [PYTHON, path_from_root('tools', 'wasm-sourcemap.py'),
                    wasm_file,
                    '--dwarfdump=' + LLVM_DWARFDUMP,
@@ -1540,7 +1574,7 @@ def check_binaryen(bindir):
 def get_binaryen_bin():
   assert Settings.WASM, 'non wasm builds should not ask for binaryen'
   global binaryen_checked
-  rtn = os.path.join(BINARYEN_ROOT, 'bin')
+  rtn = os.path.join(config.BINARYEN_ROOT, 'bin')
   if not binaryen_checked:
     check_binaryen(rtn)
     binaryen_checked = True
@@ -1572,16 +1606,12 @@ def run_binaryen_command(tool, infile, outfile=None, args=[], debug=False, stdou
       # emit some extra helpful text for common issues
       extra = ''
       # a plain -O0 build *almost* doesn't need post-link changes, except for
-      # legalization and longjmp. show a clear error for those (as the flags
-      # the user passed in are not enough to see what went wrong)
+      # legalization. show a clear error for those (as the flags the user passed
+      # in are not enough to see what went wrong)
       if shared.Settings.LEGALIZE_JS_FFI:
         extra += '\nnote: to disable int64 legalization (which requires changes after link) use -s WASM_BIGINT'
-      if shared.Settings.SUPPORT_LONGJMP:
-        extra += '\nnote: to disable longjmp support (which requires changes after link) use -s SUPPORT_LONGJMP=0'
       if shared.Settings.OPT_LEVEL > 0:
         extra += '\nnote: -O2+ optimizations always require changes, build with -O0 or -O1 instead'
-      if shared.Settings.DISABLE_EXCEPTION_CATCHING != 1:
-        extra += '\nnote: C++ exceptions always require changes'
       exit_with_error('changes to the wasm are required after link, but disallowed by ERROR_ON_WASM_CHANGES_AFTER_LINK: ' + str(cmd) + extra)
   if debug:
     cmd += ['-g'] # preserve the debug info
@@ -1591,8 +1621,7 @@ def run_binaryen_command(tool, infile, outfile=None, args=[], debug=False, stdou
   print_compiler_stage(cmd)
   # if we are emitting a source map, every time we load and save the wasm
   # we must tell binaryen to update it
-  emit_source_map = Settings.DEBUG_LEVEL == 4 and outfile
-  if emit_source_map:
+  if Settings.GENERATE_SOURCE_MAP and outfile:
     cmd += ['--input-source-map=' + infile + '.map']
     cmd += ['--output-source-map=' + outfile + '.map']
   ret = check_call(cmd, stdout=stdout).stdout
