@@ -3,12 +3,10 @@
 # University of Illinois/NCSA Open Source License.  Both these licenses can be
 # found in the LICENSE file.
 
-from __future__ import print_function
-from .toolchain_profiler import ToolchainProfiler
 import os
 import shutil
 import logging
-from . import tempfiles, filelock
+from . import tempfiles, filelock, config, utils
 
 logger = logging.getLogger('cache')
 
@@ -76,7 +74,7 @@ class Cache(object):
       logger.debug('PID %s released multiprocess file lock to Emscripten cache at %s' % (str(os.getpid()), self.dirname))
 
   def ensure(self):
-    shared.safe_ensure_dirs(self.dirname)
+    utils.safe_ensure_dirs(self.dirname)
 
   def erase(self):
     self.acquire_cache_lock()
@@ -112,7 +110,7 @@ class Cache(object):
       if os.path.exists(cachename) and not force:
         return cachename
       # it doesn't exist yet, create it
-      if shared.FROZEN_CACHE:
+      if config.FROZEN_CACHE:
         # it's ok to build small .txt marker files like "vanilla"
         if not shortname.endswith('.txt'):
           raise Exception('FROZEN_CACHE disallows building system libs: %s' % shortname)
@@ -126,7 +124,7 @@ class Cache(object):
       self.ensure()
       temp = creator()
       if os.path.normcase(temp) != os.path.normcase(cachename):
-        shared.safe_ensure_dirs(os.path.dirname(cachename))
+        utils.safe_ensure_dirs(os.path.dirname(cachename))
         shutil.copyfile(temp, cachename)
       logger.info(' - ok')
     finally:
@@ -135,32 +133,4 @@ class Cache(object):
     return cachename
 
 
-# Given a set of functions of form (ident, text), and a preferred chunk size,
-# generates a set of chunks for parallel processing and caching.
-def chunkify(funcs, chunk_size, DEBUG=False):
-  with ToolchainProfiler.profile_block('chunkify'):
-    chunks = []
-    # initialize reasonably, the rest of the funcs we need to split out
-    curr = []
-    total_size = 0
-    for i in range(len(funcs)):
-      func = funcs[i]
-      curr_size = len(func[1])
-      if total_size + curr_size < chunk_size:
-        curr.append(func)
-        total_size += curr_size
-      else:
-        chunks.append(curr)
-        curr = [func]
-        total_size = curr_size
-    if curr:
-      chunks.append(curr)
-      curr = None
-    return [''.join(func[1] for func in chunk) for chunk in chunks] # remove function names
-
-
-try:
-  from . import shared
-except ImportError:
-  # Python 2 circular import compatibility
-  import shared
+from . import shared
